import sys
from pathlib import Path

from . import root as git_root


def entrypoint() -> Path:
    fpath: Path = Path(sys.argv[0]).absolute()
    root: Path = git_root()
    if fpath.is_relative_to(root):
        return fpath.relative_to(root)
    return fpath



from . import git, integration, plugin, utils
from ._config import BaseConfig
from ._env import ENV_PREFIX, env
from ._experiment import Experiment, current_experiment, set_current_experiment
from ._main import main
from ._start import end, start
from .git import entrypoint
from .integration import Backend, BackendNeptune, backend_factory
from .plugin import Plugin, PluginGit, PluginLogging, PluginRestic, default_plugins

__all__ = [
    "ENV_PREFIX",
    "Backend",
    "BackendNeptune",
    "BaseConfig",
    "Experiment",
    "Plugin",
    "PluginGit",
    "PluginLogging",
    "PluginRestic",
    "backend_factory",
    "current_experiment",
    "default_plugins",
    "end",
    "entrypoint",
    "env",
    "git",
    "integration",
    "main",
    "plugin",
    "set_current_experiment",
    "start",
    "utils",
]



from liblaf import cherries


class Config(cherries.BaseConfig):
    name: str = "world"


@cherries.main()
def main(cfg: Config) -> None:
    ic(f"Hello, {cfg.name}!")


if __name__ == "__main__":
    main(Config())



from __future__ import annotations

import pydantic_settings as ps

from liblaf import cherries


class Plugin(ps.BaseSettings):
    enabled: bool = True
    priority: float = 0.0

    def pre_start(self) -> None:
        if self.enabled:
            self._pre_start()

    def post_start(self, run: cherries.Experiment) -> None:
        if self.enabled:
            self._post_start(run)

    def pre_end(self, run: cherries.Experiment) -> None:
        if self.enabled:
            self._pre_end(run)

    def post_end(self, run: cherries.Experiment) -> None:
        if self.enabled:
            self._post_end(run)

    def _pre_start(self) -> None: ...
    def _post_start(self, run: cherries.Experiment) -> None: ...
    def _pre_end(self, run: cherries.Experiment) -> None: ...
    def _post_end(self, run: cherries.Experiment) -> None: ...



import functools
import inspect
from collections.abc import Callable, Sequence
from typing import ParamSpec, TypeVar

from liblaf import cherries

_P = ParamSpec("_P")
_T = TypeVar("_T")


def main(
    *,
    backend: cherries.Backend | None = None,
    enabled: bool | None = None,
    plugins: Sequence[cherries.Plugin] | None = None,
) -> Callable[[Callable[_P, _T]], Callable[_P, _T]]:
    def wrapper(fn: Callable[_P, _T]) -> Callable[_P, _T]:
        @functools.wraps(fn)
        def wrapped(*args: _P.args, **kwargs: _P.kwargs) -> _T:
            exp: cherries.Experiment = cherries.start(
                backend=backend, enabled=enabled, plugins=plugins
            )
            sig: inspect.Signature = inspect.signature(fn)
            bound_args: inspect.BoundArguments = sig.bind(*args, **kwargs)
            if len(bound_args.arguments) == 1:
                exp.log_other(
                    "cherries/config", next(iter(bound_args.arguments.values()))
                )
            elif len(bound_args.arguments) > 1:
                exp.log_other("cherries/args", bound_args.arguments)
            ret: _T = fn(*args, **kwargs)
            exp.end()
            return ret

        return wrapped

    return wrapper



from liblaf import cherries


def backend_factory(backend: str | None = None) -> cherries.Backend:
    backend = backend or cherries.env.str("BACKEND", "dummy")
    match backend:
        case "neptune":
            return cherries.BackendNeptune()
        case "dummy":
            return cherries.Backend()
        case _:
            msg: str = f"Unknown backend: {backend}"
            raise ValueError(msg)



from ._abc import Plugin
from ._default import default_plugins
from ._git import PluginGit
from ._logging import PluginLogging
from ._restic import PluginRestic

__all__ = ["Plugin", "PluginGit", "PluginLogging", "PluginRestic", "default_plugins"]



from . import github
from ._commit import DEFAULT_COMMIT_MESSAGE, commit
from ._entrypoint import entrypoint
from ._repo import github_user_repo, repo, root
from .github import permalink, user_repo

__all__ = [
    "DEFAULT_COMMIT_MESSAGE",
    "commit",
    "entrypoint",
    "github",
    "github_user_repo",
    "permalink",
    "repo",
    "root",
    "user_repo",
]
